#include <qfile.h>
#include <qtextstream.h>
#include <kdebug.h>

#include "doctypedefs.h"

#include <kapp.h>
#include <kcmdlineargs.h>

int main( int argc, char **argv )
{
  KCmdLineArgs::init( argc, argv, "doctypedefs", "", "" );
  KApplication app;

  DocTypeDefs *win = new DocTypeDefs();
  win->load( "output.xml" );

  delete win;
}


DocTypeDefs::DocTypeDefs()
{
}

DocTypeDefs::~DocTypeDefs()
{
}

bool DocTypeDefs::load( const QString &filename )
{
    QFile f( filename );

    if ( !f.open( IO_ReadOnly ) )
	return false;

    if ( !defs.setContent( &f ) ) {
      f.close();
      return false;
    }

    f.close();

    processDefs();
    return true;
}

void DocTypeDefs::processDefs()
{
    QDomNodeList dnl;

    dnl = defs.elementsByTagName( "info" );
    if ( dnl.count() )
	processInfo( dnl.item(0).toElement() );

    dnl = defs.elementsByTagName( "tags" );
    if ( dnl.count() )
	processTags( dnl.item(0).toElement() );

    dnl = defs.elementsByTagName( "entities" );
    if ( dnl.count() )
	processEntities( dnl.item(0).toElement() );
}

void DocTypeDefs::processInfo( const QDomElement &elem )
{
    QDomNode n = elem.firstChild();
    while( !n.isNull() ) {
	QDomElement e = n.toElement();
	if( !e.isNull() ) {
	    if ( e.tagName() == "publicid" )
		publicid = e.text();
	    else if ( e.tagName() == "systemid" )
		systemid = e.text();
	    else if ( e.tagName() == "name" )
		name = e.text();
	}
	n = n.nextSibling();
    }
}

void DocTypeDefs::processTags( const QDomElement &elem )
{
    QDomNode n = elem.firstChild();
    while( !n.isNull() ) {
	QDomElement e = n.toElement();
	if( !e.isNull() ) {
	    if ( e.tagName() == "tag" )
		processTag( e );
	}
	n = n.nextSibling();
    }
}

void DocTypeDefs::processTag( const QDomElement &elem )
{
    Tag t;
    t.name = elem.attribute( "name" );

    QDomNode n = elem.firstChild();
    while( !n.isNull() ) {
	QDomElement e = n.toElement();
	if( !e.isNull() ) {
	    if ( e.tagName() == "attr" ) {
		Attr a;
		a.name = e.attribute( "name" );
		t.attrs[a.name] = a;
	    }
	}
	n = n.nextSibling();
    }

    tags[ t.name ] = t;
}

void DocTypeDefs::processEntities( const QDomElement &elem )
{
    QDomNode n = elem.firstChild();
    while( !n.isNull() ) {
	QDomElement e = n.toElement();
	if( !e.isNull() ) {
	    if ( e.tagName() == "entity" ) {
		Entity ent;
		ent.name = e.attribute( "name" );
		entities[ent.name] = ent;
	    }
	}
	n = n.nextSibling();
    }
}





